#!/usr/bin/python
from __future__ import division
from operator import itemgetter
import stats
from math import *
import sys
##from pygraphviz import *

class DistanceError: pass
class MissingDMatrix(DistanceError): pass
class DistanceNotImplemented(DistanceError): pass

def invSign(bestmk,mapmk):
    xbar=stats.mean(bestmk)
    ybar=stats.mean(mapmk)
    xdev=[x-xbar for x in bestmk]
    ydev=[y-ybar for y in mapmk]
    cov=stats.sum([x*y for (x,y) in zip(xdev,ydev)])
##    var=stats.ss(xdev)
##        print cov/var
    if (cov < 0):
        return -1
    else:
        return 1
    

def manhattan(h1,m1,h2,m2):
    return abs(h2-h1)+abs(m2-m1)

def euclidian(h1,m1,h2,m2):
    return sqrt(pow((h2-h1),2)+pow((m2-m1),2))

def getGroupFromGLinks(mklinks):
    groups=[] ## groups of linked markers
    nmk=len(mklinks)
    mkgroups=[-1]*nmk ## which group each marker belongs to
    igroup=0 ## next group to create
    for mk in range(nmk):
        if (mkgroups[mk]>=0):
            curigroup=mkgroups[mk]
            curgroup=groups[curigroup]
        else: ## marker does not belong to a group yet
            curigroup=-1
            tmpgroup=[mk]
            curgroup=tmpgroup
        for linkedmk in mklinks[mk]:
            ## linked marker does not belong to a group yet
            if mkgroups[linkedmk]==-1:
                curgroup.append(linkedmk)
                if (curigroup>-1):
                    mkgroups[linkedmk]=curigroup
            else:
                ## linked marker does belong to a different group: merge
                if (mkgroups[linkedmk]!=mkgroups[mk]):
                    oldigroup=mkgroups[mk]
                    togroup=mkgroups[linkedmk]
                    ##assert(togroup>-1)
                    for l in curgroup:
                        groups[togroup].append(l)
                    for i in curgroup:
                        mkgroups[i]=togroup
                    if (oldigroup>-1):
                        groups[oldigroup]=[]
                    curigroup=togroup
                    curgroup=groups[togroup]
        if (curigroup==-1):
            for i in tmpgroup:
                mkgroups[i]=igroup
            groups.append(tmpgroup)
            igroup+=1
    groups=[g for g in groups if g!=[]]
    for g in groups:
        g.sort()
    return groups
                    

def getGLinkGroup(carte,refcarte,G=2):
    m=list(carte.map)
    h=list(refcarte.map)
    nmk=len(m)
    if (nmk!=len(h)):
        raise ValueError,"carte length (%d) != Ref carte length (%d)"\
              % (nmk,len(h))
    ## compute distance matrix
    dm=[]
    for i in range(nmk):
        dm.append([])
        h1=i
        m1=m.index(h[i])
        for j in range(i+1,nmk):
            h2=j
            m2=m.index(h[j])
            if (manhattan(h1,m1,h2,m2)<=G):
                dm[i].append(j)
    G1=getGroupFromGLinks(dm)
    return(G1)

def getBP(map1,map2):
    """
    Returns the breakpoints between map1 and map2
    Return value: (bp1,bp2)
                  bp1: BP on map1
                  bp2: BP on map2
    """
    bp1=[]
    bp2=[]
    mk1=set(map1.map)
    mk2=set(map2.map)
    mkcom=mk1.intersection(mk2)
    m1s=[x for x in map1 if x in mkcom]
    m2s=[x for x in map2 if x in mkcom]
    nmk=len(mkcom)
    for i in range(nmk-1):
        mk1=m1s[i]
        mk2=m1s[i+1]
        if abs(m2s.index(mk1)-m2s.index(mk2))!=1:
            bp1.append(i)
        mk1=m2s[i]
        mk2=m2s[i+1]
        if abs(m1s.index(mk1)-m1s.index(mk2))!=1:
            bp2.append(i)
    return (mkcom,bp1,bp2)

    
        
    
def getDistance(map,refmap=[],type="manhattan",d=None):
    m=list(map)
    nmk=len(m)
    if (refmap==[]):
        h=range(nmk)
    else:
        h=refmap
    dm=[]
    for i in range(nmk-1):
        if type=="manhattan":
            dm.append(manhattan(h[i],m[i],h[i+1],m[i+1]))
        elif type=="euclidian":
            if (d==None):
                raise MissingDMatrix
            dm.append(euclidian(0,0,d[h[i]][h[i+1]],d[m[i]][m[i+1]]))
        else:
            raise DistanceNotImplemented
    return dm

def compute_adj_prob(maps,prob):
    segments=[zip(m[:-1],m[1:]) for m in maps]
    retourne=(lambda (x,y): ((x>y) and (y,x)) or (x,y))
    pairs=[map(retourne,p) for p in segments]
    Padj={}
    for pair,p in zip(pairs,prob):
        for seg in pair:
            try:
                Padj[seg]+=p
            except:
                Padj[seg]=p
    return Padj

def compute_inv_prob(maps,prob):
    startmap=Maps.Map(range(len(maps[0]))) # by definition
    pinv=[0]*len(maps[0])
    pGroupInv={}
    for m,p in zip(maps,prob):
        ## check the overall orientation of the map
        if analysis.invSign(m,range(len(maps[0])))<0:
            m.reverse()
        carth_map=Maps.Map(m)
        map_groups=analysis.getGLinkGroup(carth_map,startmap)
        for group in map_groups:
            mks=[carth_map[i] for i in group]
            smks=[startmap.index(m) for m in mks]
            sens=analysis.invSign(group,smks)
            if (sens<0):
                try:
                    pGroupInv[tuple(group)]+=p
                except KeyError :
                    pGroupInv[tuple(group)]=p
                for mk in group:
                    pinv[mk]+=p
    return pinv,pGroupInv

def analyse(fic,type_prob='mpt'):
    mcmc=open(fic)
    start=mcmc.readline().split()[1:]
    Nmk=len(start)
    data=[l.split() for l in mcmc.readlines()]
    pmpt=[float(x[0]) for x in data] ## multipoint probabilities
    p2pt=[float(x[1]) for x in data] ## 2-pt posterior probabilities
    wis =[float(x[2]) for x in data] ## IS weights
    nbp =[  int(x[3]) for x in data] ## posterior number of bp
    maps=[map(int,x[4:]) for x in data] ## maps relative to the start map
    ibestmul = 0 
    ibest2pt = p2pt.index(max(p2pt))
    if type_prob=='mpt':
        prob=mpmt
    else:
        prob=p2pt
    #### compute adjacent probabilities
    print "Calcul des probas d'adjacence"
    padj=compute_adj_prob(maps_rel,prob)
    f=open(fic+'.padj','w')
    for pair,prob in padj.items():
        print >>f, " ".join(map(str,pair)),prob
    f.close()

    #### compute inversion probabilities
    print "Calcul des probas d'inversion"
    pinv,pGroupInv=compute_inv_prob(maps,prob)
    inv=[0]*Nmk
    f=open(fic+'.inv','w')
    for i in range(Nmk):
        print >>f, start[i],pinv[i]
    f.close()
    #### global performance results
    rez={'pmpt' : pmpt[ibestmul],
         'p2pt' : p2pt[ibest2pt],
         'bpmean'  : sum([x*y for x,y in zip(nbp,prob)]),
         'nmaps' : len(maps),
         'pinv' : pinv,
         'padj' : padj,
         'start' : start,
         'wis' : wis,
         'maps' : maps,
         'ibest2pt' : ibest2pt,
         }
    return rez

