import numpy as np

class NOrders:
    '''
    Class for computing the number of orders with K breakpoints
    from the formulas in Faraut et al. (2007)
    '''
    def __init__(self,n):
        if n < 3:
            raise ValueError
        self.nmk=n 
        self.locdim=n+1# n loc ranges from 0 to n
        self.bpdim=n # bp ranges from 0 to (n-1)
        self.O=np.ones((self.locdim,self.bpdim),dtype=np.uint64)*-1
        self.Ob=np.ones((self.locdim,self.bpdim),dtype=np.uint64)*-1
        self.Oc=np.ones((self.locdim,self.bpdim),dtype=np.uint64)*-1
        self.Ib=np.ones((self.locdim,self.bpdim),dtype=np.uint64)*-1
        self.Ic=np.ones((self.locdim,self.bpdim),dtype=np.uint64)*-1
        self.Sb=np.ones((self.locdim,self.bpdim),dtype=np.uint64)*-1
        self.Sc=np.ones((self.locdim,self.bpdim),dtype=np.uint64)*-1
        ### Initialisation
        for i in range(self.locdim):
            self.Ib[i][0]=0
            self.Sb[i][0]=2
            self.Ic[i][0]=0
            self.Sc[i][0]=0
            for k in range(i,self.bpdim):
                self.O[i][k]=0
                self.Ob[i][k]=0
                self.Oc[i][k]=0
                self.Ib[i][k]=0
                self.Sb[i][k]=0
                self.Ic[i][k]=0
                self.Sc[i][k]=0
        self.Sb[0][0]=0
        self.Sb[1][0]=0

    def getNO(self,k):
        if k > (self.nmk-1) or k < 0:
            print self.nmk,k
            raise ValueError
        return self.__f_O(self.nmk,k)

    def test(self):
        for i in range(3,self.locdim):
            for j in range(i):
                print i,j,self.__f_O(i,j)
            
    def __f_O(self,n,k):
        if n <= k:
            raise ValueError
        if k < 0:
            return 0
        self.O[n][k]=self.__f_Ob(n,k)+self.__f_Oc(n,k)
        return self.O[n][k]

    def __f_Ob(self,n,k):
        if self.Ob[n][k] == -1:
            self.Ob[n][k]=self.__f_Ib(n,k)+self.__f_Sb(n,k)
        return self.Ob[n][k]
    
    def __f_Oc(self,n,k):
        if self.Oc[n][k] == -1:
            self.Oc[n][k]=self.__f_Ic(n,k)+self.__f_Sc(n,k)
        return self.Oc[n][k]

    def __f_Ib(self,n,k):
        if self.Ib[n][k] == -1:
            self.Ib[n][k]=self.__f_Ob(n-1,k-1)+2*self.__f_Oc(n-1,k-1)
        return self.Ib[n][k]
    
    def __f_Ic(self,n,k):
        if self.Ic[n][k] == -1:
            self.Ic[n][k] = (k-1)*self.__f_O(n-1,k-1)  - self.__f_Sc(n,k-1) + (n-k)*self.__f_O(n-1,k-2);
        return self.Ic[n][k]

    def __f_Sb(self,n,k):
        if self.Sb[n][k] == -1:
            self.Sb[n][k] = self.__f_Ob(n-1,k)
        return self.Sb[n][k]

    def __f_Sc(self,n,k):
        if self.Sc[n][k] == -1:
            self.Sc[n][k] = self.__f_Ib(n-1,k) + 2*self.__f_Ic(n-1,k) + self.__f_Sc(n-1,k) + self.__f_Sb(n-1,k-1) + self.__f_Sc(n-1,k-1);
        return self.Sc[n][k]
    
        
                
