#!/usr/bin/python
from __future__ import division
from operator import itemgetter
import stats
from math import *
import sys
from pygraphviz import *

class DistanceError: pass
class MissingDMatrix(DistanceError): pass
class DistanceNotImplemented(DistanceError): pass

def invSign(bestmk,mapmk):
    xbar=stats.mean(bestmk)
    ybar=stats.mean(mapmk)
    xdev=[x-xbar for x in bestmk]
    ydev=[y-ybar for y in mapmk]
    cov=stats.sum([x*y for (x,y) in zip(xdev,ydev)])
##    var=stats.ss(xdev)
##        print cov/var
    if (cov < 0):
        return -1
    else:
        return 1
    

def manhattan(h1,m1,h2,m2):
    return abs(h2-h1)+abs(m2-m1)

def euclidian(h1,m1,h2,m2):
    return sqrt(pow((h2-h1),2)+pow((m2-m1),2))

def getGroupFromGLinks(mklinks):
    groups=[] ## groups of linked markers
    nmk=len(mklinks)
    mkgroups=[-1]*nmk ## which group each marker belongs to
    igroup=0 ## next group to create
    for mk in range(nmk):
        if (mkgroups[mk]>=0):
            curigroup=mkgroups[mk]
            curgroup=groups[curigroup]
        else: ## marker does not belong to a group yet
            curigroup=-1
            tmpgroup=[mk]
            curgroup=tmpgroup
        for linkedmk in mklinks[mk]:
            ## linked marker does not belong to a group yet
            if mkgroups[linkedmk]==-1:
                curgroup.append(linkedmk)
                if (curigroup>-1):
                    mkgroups[linkedmk]=curigroup
            else:
                ## linked marker does belong to a different group: merge
                if (mkgroups[linkedmk]!=mkgroups[mk]):
                    oldigroup=mkgroups[mk]
                    togroup=mkgroups[linkedmk]
                    assert(togroup>-1)
                    for l in curgroup:
                        groups[togroup].append(l)
                    for i in curgroup:
                        mkgroups[i]=togroup
                    if (oldigroup>-1):
                        groups[oldigroup]=[]
                    curigroup=togroup
                    curgroup=groups[togroup]
        if (curigroup==-1):
            for i in tmpgroup:
                mkgroups[i]=igroup
            groups.append(tmpgroup)
            igroup+=1
    groups=[g for g in groups if g!=[]]
    for g in groups:
        g.sort()
    return groups
                    

def getGLinkGroup(map,G=2,refmap=[]):
    m=list(map)
    nmk=len(m)
    if (refmap==[]):
        h=range(1,nmk+1)
    else:
        h=refmap
    ## compute distance matrix
    dm=[]
    for i in range(nmk):
        dm.append([])
        for j in range(i+1,nmk):
            if (manhattan(h[i],m[i],h[j],m[j])<=G):
                dm[i].append(j)
    return(getGroupFromGLinks(dm))

    
def getDistance(map,refmap=[],type="manhattan",d=None):
    m=list(map)
    nmk=len(m)
    if (refmap==[]):
        h=range(nmk)
    else:
        h=refmap
    dm=[]
    for i in range(nmk-1):
        if type=="manhattan":
            dm.append(manhattan(h[i],m[i],h[i+1],m[i+1]))
        elif type=="euclidian":
            if (d==None):
                raise MissingDMatrix
            dm.append(euclidian(0,0,d[h[i]][h[i+1]],d[m[i]][m[i+1]]))
        else:
            raise DistanceNotImplemented
    return dm


##

def merge_graph(ni,nj,G,N,E):
    gi = N[ni] ## target graph
    gj = N[nj] ## source graph
    print (gi,gj)
    for n in G[gj]:
        G[gi]+=(n,)
        N[n]=gi
    for e in [el for el in E if E[e]==gj]:
        E[e]=gi
    del(G[gj])

    

def getMlinkGroups(Mlink,cutoff=0.99):
    """
    Determines linkage groups with marker links > cutoff \n

    Arguments:
    Mlink : for each marker, gives link probability with the next ones.\n
    Upper Triangular matrix (nmk-1)x(nmk-1).
    . . . . . . \n
    . . . . . \n
    . . . . \n
    . . . \n
    . . \n
    . \n
    
    """
    Graphs={}
    Noeuds={}
    Edges={}
    ig=-1

    rel=[]
    nmk=len(Mlink)+1
    for mk1 in range(nmk-1):
        for mk2 in range(nmk-1-mk1):
            if (Mlink[mk1][mk2] > cutoff):
                rel.append((mk1,mk2+mk1+1))

    print "Found "+str(len(rel))+" relations\n"
    for r in rel:
        if not (Noeuds.has_key(r[0]) or Noeuds.has_key(r[1])):
            ig+=1
            Graphs[ig]=(r[0],r[1])
            Noeuds[r[0]]=ig
            Noeuds[r[1]]=ig
            Edges[r]=ig
        elif Noeuds.has_key(r[0]):
            if Noeuds.has_key(r[1]):
                if (Noeuds[r[0]]!=Noeuds[r[1]]):
                    if (Noeuds[r[0]]>Noeuds[r[1]]):
                        merge_graph(r[0],r[1],Graphs,Noeuds,Edges)
                    else:
                        merge_graph(r[1],r[0],Graphs,Noeuds,Edges)
            else:
                Noeuds[r[1]]=Noeuds[r[0]]
                Graphs[Noeuds[r[0]]]+=(r[1],)
                Edges[r]=Noeuds[r[1]]
        else: ## Noeuds.has_key[r[1]]
            Noeuds[r[0]]=Noeuds[r[1]]
            Graphs[Noeuds[r[1]]]+=(r[0],)
    return (Graphs,Edges)

                         

def plotMlink(Mlink,cutoff=0.5,output="mcmcgraph",dist=None,format='ps',structured=True):
    """
    Create a .dot file for graphviz system to produce a graph\n

    Arguments:
    Mlink: adjacence probabilities
    cutoff: edges are drawn if p(adj)>cutoff. By default all non null edges are drawn.
    dist: rh 2pt breakage probabilities
    """
    eps=0.0001
    G=AGraph(directed=False)
    G.graph_attr['label']="Adjacence Graph"
    G.graph_attr['orientation']='landscape'
    G.node_attr['shape']='plaintext'
    G.node_attr['pointsize']='50'
    
#     G.node_attr['height']="0.1"
#     G.node_attr['width']="0.1"
    
    nmk=len(Mlink)+1
    for mk1 in range(nmk-1):
        for mk2 in range(nmk-1-mk1):
            if (Mlink[mk1][mk2] > 0):
                # adjust small weights
                w=max(0.0001,Mlink[mk1][mk2])
                if (dist != None):
                    l=min(dist[mk1][mk2+mk1+1],0.99999)
                    l=max(abs(log(1-l)),eps)
                else:
                    l=1
                if (w > cutoff):
                    G.add_edge(mk1,mk2+mk1+1)
                    e=G.get_edge(mk1,mk2+mk1+1)
                ## For high probabilities, draw pretty edges
                    e.attr['len']=str(l)
                    if (w > 0.99):
                        e.attr['color']='red'
                        e.attr['style']='bold'
    if structured:
        ## impose a structure on the graph with invible edges
        mk=100
        while (mk < nmk):
            G.add_edge(mk-100,mk)
            e=G.get_edge(mk-100,mk)
            e.attr['style']='invis'
            mk+=100
    G.write(output+".dot")
    print "Graph structure wrote to file "+output+".dot"
    G.layout()
    G.draw(output+'.'+format)
    print "Graph image wrote to file "+output+"."+format
    
