import MapsIO
import copy #Je ne l'utilise qu'une seule fois et c'est vraimant necessaire
import Tools
import Groups

#Common intervals package build with the article of Anne Bergeron : Computing common intervals of K permutations
# abbreviation : CI = Common Interval


#For a permutation Imax for one element of the permutation is the largest interval that contain this element and not element smaller
#		Imin for one element of the permutation is the largest interval that contain this element and not element greather
#		Sup of one element is the largest integer such that the interval between this element and  sup of this element is include 	
#	in the IMax of this element
#		Inf of one element is the smallest integer such that the interval between inf of this element and this element  is include  
#		in the IMax of this element

#############################################################################################################################################      Left and Right bounds for Imax and Imin  a permutation P (The referency permutation is the identity permutation Idn (0,1,2,..n)      
###########################################################################################################################################



def left_bound_Imax (permutation) :
	'''
	INPUT : The permutation in a list form
	OUTPUT : a dictionary with the left bound of the Imax for each component of the permutation. This dictionary has as key each component of the permutation and as contain the left bound of the Imax.
	'''
	S=[]
	S.append(0)
	LMax={}
	perm=permutation[:]
	perm=[-1]+perm
	for i in range(1,len(perm)) :
		while perm[i]<perm[S[-1]]:
			del S[-1]
		LMax[perm[i]]=S[-1]
		S.append(i)
	return LMax




def right_bound_Imax (permutation) :
	'''
	INPUT : The permutation in a list form
	OUTPUT : a dictionary with the right bound of the Imax for each component of the permutation. This dictionary has as key each component of the permutation and as contain the right bound of the Imax.
	'''
	S=[]
	S=[len(permutation)]
	RMax={}
	perm=permutation[:]
	perm.append(-1)
	for i in range(len(perm)-2,-1,-1) :
		while perm[i]<perm[S[-1]]:
			del S[-1]
		RMax[perm[i]]=S[-1]-1
		S.append(i)
	return RMax




def left_bound_Imin (permutation) :
	'''
	INPUT : The permutation in a list form
	OUTPUT : a dictionary with the left bound of the Imin for each component of the permutation. This dictionary has as key each component of the permutation and as contain the left bound of the Imin.
	'''
	S=[]
	S.append(0)
	LMax={}
	perm=permutation[:]
	perm=[len(permutation)]+perm
	for i in range(1,len(perm)):
		while perm[i]>perm[S[-1]]:
			del S[-1]
		LMax[perm[i]]=S[-1]
		S.append(i)
	return LMax

def right_bound_Imin (permutation) :
	'''
	INPUT : The permutation in a list form
	OUTPUT : a dictionary with the right bound of the Imin for each component of the permutation. This dictionary has as key each component of the permutation and as contain the right bound of the Imin.
	'''
	S=[]
	S=[len(permutation)]
	RMax={}
	perm=permutation[:]
	perm.append(len(permutation))
	for i in range(len(perm)-2,-1,-1) :
		while perm[i]>perm[S[-1]]:
			del S[-1]
		RMax[perm[i]]=S[-1]-1
		S.append(i)
	return RMax

###########################################################################################################################################
########                                              Inverse Permutation                                                        ##########
###########################################################################################################################################

def inverse_permutation (permutation) :
	'''
	INPUT : The permuation in a list form
	OUTPUT: the inverse permutation in a dictionary form, the keys are the positions and the content the elements of the permutation.
	'''
	inverse={}
	for i in range(len(permutation)):
		inverse[i]=permutation.index(i)
	return inverse


##########################################################################################################################################
###################                                  Generator (Sup,Inf)                                      ##############################
############################################################################################################################################

def generator(r_Imin,l_Imin,r_Imax,l_Imax,perm,inv_perm):
	'''
	INPUT : the right and left bounds for Imin and Imax, the permutation in a list form and the inverse permuation in a dictionary form
	OUTPUT : A list with two dictionary : the sup and the inf dictionary each one associate a component of the permutation to its sup or inf
	'''
	Inf={0:0}
	Sup={len(perm)-1:len(perm)-1}
	m={}
	M={}
	for k in range(len(perm)):
		m[k]=k
		M[k]=k
	for k in range(1,len(perm)):
		while m[k]!=0 and inv_perm[m[k]-1]>=l_Imin[k] and inv_perm[m[k]-1]<=r_Imin[k]:
			temporate=m[k]
			del m[k]
			m[k]=m[temporate-1]
		Inf[k]=m[k]
	for k in range(len(perm)-2,-1,-1):
		while M[k]!=len(perm)-1 and inv_perm[M[k]+1]>=l_Imax[k] and inv_perm[M[k]+1]<=r_Imax[k] :
			temporate=M[k]
			del M[k]
			M[k]=M[temporate+1]
		Sup[k]=M[k]
	return [Inf,Sup]	




############################################################################################################################################
#########################         Support for the commuting generator( Sup,Inf)                                          ###################
############################################################################################################################################

def support(Sup):
	''' 
	INPUT : The sup generator
	OUTPUT : a dictionary with the support for each component of the permutation
	'''
	S=[0]
	support={}
	for i in range(1,len(Sup)):
		while Sup[S[-1]]<i:
			del S[-1]
		support[i]=S[-1]
		S.append(i)
	return support



############################################################################################################################################
##################          Common interval for the set of permuation{Idn and P}                                           #################
############################################################################################################################################


def common_interval(support,Inf):
	'''
	INPUT : the support dictionary, the Inf generator
	OUPUT : A list with all the CI
	'''
	CI=[]
	for j in range(len(Inf)-1,-1,-1) :
		i=j
		end=False
		while i>=Inf[j] and not end:
			CI.append(range(i,j+1))
			if i>0 :
				i=support[i]
			else:
				end=True
	return CI



#----------------------------------------------------------------------------------------------------------------------------------------
# 			GENERATOR FOR THE ENCODED MAPS
#----------------------------------------------------------------------------------------------------------------------------------------

def GetMapsGenerator(maps_encoded) :
	'''
	 we compute the Inf and Sup generator for the maps. that is the same as computing the generator for k permutation,  to do this we  compute the generator of each permutation and we use the propriety of the union of generators
	INPUT : maps_encoded : the distribution with as first map the identity maps(0...n)
	OUTPUT : the [Inf,Sup]generator
	'''
	Inf={}
	Sup={}
	identity=list(maps_encoded[0].map)
	
	
	for i in range(1,len(maps_encoded)):
		my_permutation=list(maps_encoded[i].map)
		lbmax=left_bound_Imax (my_permutation)
		rbmax=right_bound_Imax (my_permutation)
		lbmin=left_bound_Imin (my_permutation)
		rbmin=right_bound_Imin (my_permutation)
		inv_perm=inverse_permutation(my_permutation)
		[Inf_perm,Sup_perm]=generator(rbmin,lbmin,rbmax,lbmax,my_permutation,inv_perm)
		if Inf!={} and Sup!={}:
			for i in identity:
				Inf[i]=max(Inf[i],Inf_perm[i])
				Sup[i]=min(Sup[i],Sup_perm[i])
		else:
			[Inf,Sup]=[Inf_perm,Sup_perm]
	return [Inf,Sup]



#-------------------------------------------------------------------------------------------------------------------------------------------
#                                         FIND ALL THE CI FOR A GIVEN MAPS DISTRIBUTION                               
#-------------------------------------------------------------------------------------------------------------------------------------------

def GetCIs(maps):
	'''
	INPUT : maps= list of all the maps of the distribution
	OUTUP : all the common intervals 
	'''
	#We need to encode the ditribution to have the identity premutation (0..n) as first map
	maps_encoded=copy.deepcopy(maps)
	coding=MapsIO.RecodeMaps(maps_encoded)
	#We buil the generator
	[Inf,Sup]=GetMapsGenerator(maps_encoded)
	#We build the support
	my_support=support(Sup)
	#We build the coded Common Intervals 
	encoded_CI=common_interval(my_support,Inf)
	#We uncode all the Common Intervals
	CIs=[]
	for a_CI in encoded_CI:
		CIs.append([coding['new2old'][item] for item in a_CI])
	return CIs	



#-------------------------------------------------------------------------------------------------------------------------------------------
#								STRONG COMMON INTERVALS 				
#-------------------------------------------------------------------------------------------------------------------------------------------
def Lsupport(L):
	''' 
	INPUT : The L generator
	OUTPUT : a dictionary with the support for each component of the permutation
	'''
	S=[len(L)-1]
	support={}
	for i in range(len(L)-2,-1,-1):
		while L[S[-1]]>i:
			del S[-1]
		support[i]=S[-1]
		S.append(i)
	return support


def canonical_generator(Rp,Lp,Common_Intervals):
	''' 
	INPUT : Non canonical generator Rp and Lp, the common intervals
	OUTPUT : the canonical generator
	'''
	R={}
	L={}
	Rp_support=support(Rp)
	Lp_support=Lsupport(Lp)
	R[0]=len(Rp)-1
	L[len(Lp)-1]=0
	for k in range(1,len(Rp)):
		R[k]=k
	for k in range(0,len(Rp)-1):
		L[k]=k
	for k in range(len(Rp)-1,0,-1):
		if range(Rp_support[k],R[k]+1) in Common_Intervals:
			R[Rp_support[k]]=max(R[k],R[Rp_support[k]])
	for k in range(0,len(Rp)-1):
		if range(L[k],Lp_support[k]+1) in Common_Intervals:
			L[Lp_support[k]]=min(L[k],L[Lp_support[k]])
	return[R,L]

def find_strong_intervals(R,L):
	'''
	INPUT : R and L : the canoncial generator
	OUPUT : A list with all the strong  CI
	'''

	
	strong_ICs=[]
	S=[]
	left_bounds=sorted(L.values()+R.keys())
	right_bounds=sorted(R.values()+L.keys())
	a=sorted(left_bounds+right_bounds)
	left_bounds_index=[]
	for x in left_bounds :
		my_index=a.index(x) 
		while my_index in left_bounds_index :
			my_index=my_index+1
		left_bounds_index.append(my_index)
	for i in range(len(a)):
		if i in left_bounds_index :
			S.append(a[i])
		else :
			if range(S[-1],a[i]+1) not in strong_ICs:
				strong_ICs.append(range(S[-1],a[i]+1))
			if len(S) !=1 :
				del S[-1]
	return strong_ICs


def GetStrongCIs(maps):
	'''
	INPUT : maps= list of all the maps of the distribution
	OUTUP : all the common intervals 
	'''
	#We need to encode the ditribution to have the identity premutation (0..n) as first map
	maps_encoded=copy.deepcopy(maps)
	coding=MapsIO.RecodeMaps(maps_encoded)
	#We buil the generator
	[Inf,Sup]=GetMapsGenerator(maps_encoded)
	#We build the Sup Genrator
	my_support=support(Sup)
	#We build the coded Common Intervals 
	encoded_CI=common_interval(my_support,Inf)
	#new generator
	[R,L]=canonical_generator(Sup,Inf,encoded_CI)
	#Strong CIs :
	strong_CIs_encoded=find_strong_intervals(R,L)
	#We uncode all the Common Intervals
	strong_CIs=[]
	for a_strong_CI in strong_CIs_encoded:
		strong_CIs.append([coding['new2old'][item] for item in a_strong_CI])
	return strong_CIs




#--------------------------------------------------------------------------------------
#                                         BUILD TREE                      
#--------------------------------------------------------------------------------------
def BuildCommonIntervals (my_partition,all_strong_CI) :
	'''
	Create a new branch in the tree
	INOUT : my_partition
	IN : the strong common intervals that correspond to the actual upper nodes of the tree
	OUT : the updated CIs 
	'''
	# Algorithm :
	#Find the common interval taht have a size taller than 1
	#Create a new group
	#Update the list of CI
	use_CI=[x for x in all_strong_CI if len(x)>1]
	size_CI=[len(x) for x in use_CI]
	smallest_CI=use_CI[size_CI.index(min(size_CI))]
	#5
	new_part=[]
	for x in smallest_CI :
		new_part.append(my_partition[x])
		del my_partition[x]
	new_part_group=Groups.Group(new_part)
	
	#save the father
	for x in  new_part_group.group:
		x.father=new_part_group

	my_partition[new_part_group.code]=new_part_group
	UpdateCIs(new_part_group,all_strong_CI)



def UpdateCIs(new_group,CIs) :
	'''
	Code the a group of component of the common intervals
	INPUT : new_group : the new node of the tree
	INOUT : the common intervals for the upper nodes of the tree
	'''
	
	for my_CI in [x for x in CIs if len(x)>1]:
		
		if [x for x in  my_CI if x in new_group.show_components()] !=[] :
			new_group_index=[my_CI.index(x) for x in new_group.show_components()]
			my_CI[min(new_group_index)]=new_group.code
			for x in range(min(new_group_index)+1,max(new_group_index)+1):
				del my_CI[min(new_group_index)+1]
