from metamap import stats
import Maps
import Groups
import utiles
import IO

#######################################################################################
#               Maps File management  and genome assembly file management            # #######################################################################################


# Maps file : the file with the maps distribution. 
#		The first line can be the name of all the markers (the position of each one correspond to its code in the rest of the file) or 			a code associate to the name of the marker.
#		In the other line the first nulber is the probabilioty of the conciderate map from the 4th position to the end it is the map

# Assembly file : from the line two to the end, each second word of the line is the name of a marker.

#######################################################################################
#####				     MANEGEMENT OF THE MAPS FILE              	 ######
#######################################################################################


def RewriteMapsFile(use_file,coding_file,name_new_file) :
	'''
	this function permit to tranform the firt line of the maps ditribution file when we don't have the name of the markers
	INPUT : use_file: the file with the map distribution
		coding_file : the file that permit to associate the number of the first line of use_file to the name of the markers if the file doesn't contain the name of the markers
	OUTPUT : the new file named name_new_file
	'''	

	file = open(use_file,'r')
	data=file.readlines()
	line_marker_code=data[0].split()[2:]
	file_code= open(coding_file,'r')
	code_data=file_code.readlines()
	dictionary={}
	for line in code_data :
		split_line=line.split()
		dictionary[split_line[0]]=split_line[1]
	first_line=[dictionary[x] for x in line_marker_code]
	new_file=open(name_new_file,'w')
	new_file.write('startmap: ')
	for marker in first_line :
		new_file.write(marker+' ')
	new_file.write('\n')
	for line in data[1:] :
		new_file.write(line)
	file.close()



def ReadMapsFile(file,iprob,nmaps=None,reverse=False,mklist=None):
	'''
	Read the map distribution from file *file*, map posterior probability is either the multipoint prob (iprob=0) 
	or the two-point prob (iprob=1). 
	nmaps=N allows to read only the first N maps
	reverse=True specifies that all maps should be inverted
	mbeg,mend: optional, first and last marker (excluded) to read in the maps
	returns: a list of Map objects , a list of marker names
	'''	
	mcmc = open(file)
	# first line gives mk names
	mknames=mcmc.readline().split()[1:]
	if mklist is not None:
		mklist=[m for m in mklist if m in mknames]
		subset=[mknames.index(m) for m in mklist]
	else:
		subset=range(len(mknames))
	# then maps 
	il=0
	maps=[]
	for ligne in mcmc:
		buf=ligne.split()
		if float(buf[iprob])==0:
			continue
		il+=1
		if reverse:
			order=map(int,buf[4:][::-1])
		else:
			order=map(int,buf[4:])
		order=[i for i in order if i in subset]
		maps.append(Maps.Map( tuple(order) , float(buf[iprob])) )
		if nmaps is not None and (il > nmaps):
			break
	###### Below is shorter but memory consuming
	#data=[l.split() for l in mcmc.readlines()[:nmaps]] ## memory hog
	## map(int,x[4:]) returns the mk positions
	## [::-1] returns a reversed list
# 	if reverse:
# 		maps=[Maps.Map( tuple(map(int,x[4:][::-1])) , float(x[iprob]) ) for x in data]
# 	else:
# 		maps=[Maps.Map( tuple(map(int,x[4:])) , float(x[iprob]) ) for x in data]
	return maps,mknames




def ReorienteMaps(maps):
	'''
	Search for maps globally inverted (negative regression coeff) compared to the first map
	and reverse them.
	'''
	xbar=stats.mean(maps[0].map)
	xdev=[x-xbar for x in maps[0].map]
	ybar=xbar

	meanx=float(sum(maps[0])/float(len(maps[0])))
	meany=meanx
	x=sorted(maps[0])

	for m in maps[1:]:
		ydev=[y-ybar for y in m.map]
		cov=stats.sum([x*y for (x,y) in zip(xdev,ydev)])
		if cov<0:
			L=list(m.map)
			L.reverse()
			m.map=tuple(L)

def RecodeMaps(all_maps):
	'''
	Permit to encode the map distribution to have as more probable map the identity map coded from 0 to n 
	Returns the coding matches 'new2old' and 'old2new'
	'''
	coding={ 'new2old':{},'old2new':{} }
	best_map=all_maps[0] ## explicit is better than implicit
	for i in range(len(best_map)):
		coding['old2new'][best_map[i]]=i
		coding['new2old'][i]=best_map[i]
	for my_map in all_maps:
		my_map.map=tuple([coding['old2new'][j] for j in my_map.map]) 
	return coding




def CodeMarkers(marker_names,coding):
	'''
	Build Marker Objects that will be used as the leaves of the metamap tree
	marker_names: markers ordered according to the start map
	returns Marker objects stored in a Dictionnary with codes as keys
	'''
	## Do we really need a dictionnary
	markers={}
	for i in coding['new2old'].keys():
		markers[i]=Groups.Marker(i,marker_names[coding['new2old'][i]])	
	return markers




def NormaliseDistribution(maps) :
	'''
	Recompute map probabilities so that they sum up to 1
	'''
	tot_prob=stats.sum([x.prob for x in maps])
	for m in maps:
		m.prob=m.prob/tot_prob
	return tot_prob





def LoadMaps(maps_file,column_proba,number_maps=None,reverse=False,mklist=None):
	'''
	Read maps from a MCMC output file and return a list of maps and marker coding
	'''
	maps,marker_names=ReadMapsFile(maps_file,column_proba,number_maps,reverse=reverse,mklist=mklist)
	#make sure the sum of posterior map probabilities sum up to 1
	sum_prob=NormaliseDistribution(maps)
	#Reoriente the maps that need to be reoriented
	ReorienteMaps(maps)
	#Recode the maps to have the more probable map (the first one in the maps list) as the identity permutation
	coding=RecodeMaps(maps)
	
	#Put the vector of the marker name in order, to obtain a vector where the code in the distribution is equivalent to the index in the vector
	markers=CodeMarkers(marker_names,coding)
		
	return maps,markers




#######################################################################################
######                      MANEGEMENT OF THE ASSEMBLAGE FILE 			 ######
#######################################################################################


def LoadGenomeAssembly(genome_assembly_file,markers):
	''' 
	this function permit to create the assembly  map with the marker object
	INPUT : markers: the dictionary with as key the code of the marker and as contain the marker object
		assembly_map : the map with the marker name  
	OUTPT : the genome assembly memorized in a group
	'''
	assembly=IO.ParseInput(genome_assembly_file)
	assembly_map=assembly.getmap()
	
	#we creat a dictionary that associate maker names and marker objects
	all_marker_sorted_by_name={}
	for i in markers :
		all_marker_sorted_by_name[markers[i].name]=markers[i]
	
	#We read the genome assembly file and we memorise in a list the ordonned marker objects
	genome_assembly=[]
	for marker in assembly_map.map:
		genome_assembly.append(all_marker_sorted_by_name[marker])
		
	return (Groups.Sequence(genome_assembly), assembly.position)
