import logging
from copy import copy

LEVELS = {'debug': logging.DEBUG,
          'info': logging.INFO,
          'warning': logging.WARNING,
          'error': logging.ERROR,
          'critical': logging.CRITICAL}


class Metamap():
    '''
    Metamaps are inclusion trees that summarize the uncertainty present
    in a marker ordering. 
    '''
    def __init__(self,name='Metamap'):
        '''
        Initialize the Metamap with a name
        '''
        self.name=name
        
    def build(self,infile,
              mklist=None,
              twopoint=True,
              nmaps=None,
              reverse=False,
              logger=None
              ):
        '''
        Build a metamap from the map distribution stored in file
        infile. Options are:
        - mklist: only consider the list of markers (names) in mklist
        - twopoint: use twopoint probability or multipoint if  False
        - nmaps: only read the first n maps.
        - reversed : reverse all maps.
        - logger: use the logger instance as a logger (default: create one)
        '''
        ## First set up a Logger to show what we are doing
        if logger is None:
            LOG_FILE_NAME=self.name+'Metamap.log'
            logging.basicConfig(level=logging.DEBUG,
                                format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
                                datefmt='%m-%d %H:%M',
                                filename=LOG_FILE_NAME,
                                filemode='w')
            console = logging.StreamHandler()
            console.setLevel(LEVELS['debug'])
            formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
            console.setFormatter(formatter)
            logging.getLogger('').addHandler(console)
            self.logger=logging.getLogger('Metamap')
        else:
            self.logger=logger
        self.logger.info('Loading Map Distribution')
        self.maps,self.markers=MapsIO.LoadMaps(infile,int(twopoint),nmaps,reverse,mklist)
        self.logger.info('Read %d maps with %d markers' % len(self.maps),len(self.markers))
        ## est-ce vraiment bien necessaire
        self.Groups=copy(markers)
