import Groups
import platform
from os import mkdir,system,environ
from sys import stdout
from operator import itemgetter
from math import log
import yapgvb

###
if not environ.has_key('GVBINDIR'):
    pfmt=platform.system()
    if pfmt == 'Linux':
        environ['GVBINDIR']='/usr/lib/graphviz'

##import networkx as nx

class Report():
    def __init__(self,project='Metamap'):
        self.title='MetaMap Analysis' 
        self.head=\
'''<?xml version="1.0"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
<head>
<meta http-equiv="Content-Type" content="text/html;charset=utf-8" />
<meta name="generator" content="MetaMap" />
<style type="text/css">
h1 { text-align : center ; background : green ; color : yellow}
p#banner { text-align : center }
.group { border : 1px ; border-style : dotted ; width=1200}
.tree img {
        max-width: 1000px
       }

</style>
<title>%s</title>
</head><body>''' % self.title
        self.banner='<p id="banner"> %s </p>' % self.title
        self.foot='</body></html>'
        self.content=''
    def write(self,stream=stdout):
        print >> stream,self.head
        print >> stream,self.banner
        print >> stream,self.content
        print >> stream,self.foot
    def append(self,data):
        self.content+=str(data)+'\n'

def node_color(grp):
    return (grp.type=='sequence' and 'blue') or (grp.type=='marker' and 'red') or 'black'

def node_bgcolor(grp):
    return (grp.IsMkSeq() and 'green') or 'white'

def node_name(grp):
    if grp.type=='marker':
        return str(grp.code)
    return str(min(grp.show()))+'-'+str(max(grp.show()))+'\\n #'+str(len(grp.show()))

def node_ref(grp):
    if grp.type=='marker':
        return str(grp.code)
    return str(min(grp.show()))+'-'+str(max(grp.show()))

## Build a 

def build_tree(partition,show_mk=False,name='tree',base_URL=''):
    tree=yapgvb.Digraph(name)
    ## If we have multiple groups, create an artificial root joining everyone
    if (len(partition) > 1):
        grp=Groups.Group(partition.values())
        my_partition={grp.code:grp}
    else:
        my_partition=partition
    rootcode=my_partition.keys()[0]
    root=[my_partition[rootcode]]
    root_nodes=[tree.add_node(label=node_name(root[0]))]
    nmk=len(my_partition[rootcode].show())
    iter=1
    while True:
        new_root=[]
        new_root_nodes=[]
        new_nodes=[]
        n_groups=0
        for ifa in range(len(root)):
            father=root[ifa]
            father_node=root_nodes[ifa]
            ## if we're at a marker we have nothing to do
            if father.type=='marker':
                continue
            ## IF all subgroups are markers, we're done too (not going to draw THAT)
            proceed=False
            for x in father.group:
                if not x.type=='marker':
                    proceed=True
                    break
            if not proceed:
                continue
            ## if we're on an internal node, let's go
            n_groups+=1
            if show_mk:
                new_nodes=[tree.add_node(label=node_name(x),color=node_color(x),fillcolor=node_bgcolor(x)) for x in father.group]
                new_root+=[i for i in father.group]
            else:
                new_nodes=[tree.add_node(label=node_name(x),color=node_color(x),fillcolor=node_bgcolor(x))
                           for x in father.group if not x.type=='marker']
                new_root+=[x for x in father.group if not x.type=='marker']

            for node in new_nodes:
                edge=node<<father_node
            new_root_nodes+=new_nodes
        if n_groups==0:
            break
        root=new_root
        root_nodes=new_root_nodes
        iter+=1
    return tree

def txt_tree(partition,show_mk=False):
    buf=""
    if (len(partition) > 1):
        grp=Groups.Group(partition.values())
        my_partition={grp.code:grp}
    else:
        my_partition=partition
    rootcode=my_partition.keys()[0]
    root=[my_partition[rootcode]]
    nmk=len(my_partition[rootcode].show())
    iter=1
    while True:
        new_root=[]
        n_groups=0
        tw='Depth '+str(iter)
        for ifa in range(len(root)):
            father=root[ifa]
            ## if we're at a marker we have nothing to do
            if father.type=='marker':
                continue
            ## IF all subgroups are markers, we're done too (not going to draw THAT)
            proceed=False
            for x in father.group:
                if not x.type=='marker':
                    proceed=True
                    break
            if not proceed:
                continue
            n_groups+=1
            for x in father.group:
                tw+='\t'+str(min(x.show()))+'-'+str(max(x.show()))+'(#'+str(len(x.show()))+')'
            ## if we're on an internal node, let's go
            if show_mk:
                new_root+=[i for i in father.group]
            else:
                new_root+=[x for x in father.group if not x.type=='marker']
        if n_groups==0:
            break
        buf+=tw+'\n'
        root=new_root
        iter+=1
    return buf

def get_width(fictree):
    f=open(fictree)
    for ligne in f:
        buf=ligne.split()
        if buf[0]=='<svg':
            break
    w=buf[1][7:]
    return min(int(w[:-3]),1200)
        
def WriteReport(partition,prefix='metamap'):
    ## filename=prefix+'.html'
    wd=prefix+'/'
    url=wd+'index.html'
    try:
        mkdir(wd)
    except OSError:
        pass
    try:
        mkdir(wd+'/pics')
    except OSError:
        pass
    
    ## Title
    report=Report(prefix)
    ## Main Tree
    report.append('<h1><a name="top" id="top"></a>Top Level Map</h1>\n')
    ## Build sub-trees of top level
    rootcode=partition.keys()[0]
    root=[partition[rootcode]]
    nTopGroups=len(root[0].group)
    report.append('<div class="section">')
    report.append('The Top Level MetaMap is composed of %d groups<br/>'%nTopGroups)
    report.append(" <br/> \n".join(['<a href="#%s"> %s </a>' % (node_ref(x),node_ref(x)) for x in root[0].group]))
    report.append("<h2>Possible ordering of top level groups:</h2>")
    for order in sorted(root[0].possible_order.items(),key=itemgetter(1),reverse=True):
        report.append('<br/>'+" ".join([ "%5d" % x for x in order[0]])+(': <b> %f</b>'% order[1]))
    ##
    report.append("</div>") ## top level section
    tree=build_tree(partition,show_mk=True)
    tree.layout(yapgvb.engines.dot)
    tree.render(wd+'/pics/metamap.png')
    ## groups
    report.append('<div class="section">')
    report.append("<h1> Groups Description </h1>")
    children=[]
    for ig in range(nTopGroups):
        tree_name='tree'+str(ig+1)+'.png'
        grp=root[0].group[ig]
        if not grp.type=='marker':
            children+=[x for x in grp.group if not x.type=='marker']
        else:
            continue
        ref=node_ref(grp)
        tree=build_tree({ig: grp },show_mk=True,base_URL=url)
        tree.layout(yapgvb.engines.dot)
        tree.render(wd+'/pics/'+tree_name)
        report.append(\
'''
<div class="group">
<h2><a name="%s" id="g%s">Group %s</a></h2> <a href="#top">[back to Top]</a> <br/> 
<div class="tree"><img src="%s"></img></div>
''' %(ref,ref,ref, 'pics/'+tree_name))
        report.append('<div class="ordering">')
        report.append('<h3> Possible ordering of subgroups:</h3>')
        if not grp.type=='marker':
            for order in sorted(grp.possible_order.items(),key=itemgetter(1),reverse=True):
                report.append('<br/>'+" ".join([ "%5d" % x for x in order[0]])+(': <b> %f</b>'% order[1]))
        report.append('</div><br/>')## ordering
        report.append("Subgroups: "+" / ".join(['<a href="#%s"> %s </a>' % (node_ref(x),node_ref(x)) for x in grp.group if not x.type=="marker"])+"<br/>\n")
        report.append('</div>')## group class

    while len(children)!=0:
        newchildren=[]
        for grp in children:
            newchildren+=[x for x in grp.group if not x.type=='marker']
            ref=node_ref(grp)
            tree_name='tree'+ref+'.png'
            tree=build_tree({ref: grp },show_mk=True,base_URL=url)
            tree.layout(yapgvb.engines.dot)
            tree.render(wd+'/pics/'+tree_name)
            report.append(\
'''
<div class="group">
<h2><a name="%s" id="g%s">Group %s</a></h2> <a href='#%s'> [Up one level] </a> <a href="#top">[back to Top]</a> <br/>
<div class="tree"><img src="%s"></img></div> 
'''%(ref,ref,ref,node_ref(grp.father), 'pics/'+tree_name))
            report.append('<div class="ordering">')
            report.append('<h3> Possible ordering of subgroups:</h3>')
            for order in sorted(grp.possible_order.items(),key=itemgetter(1),reverse=True):
                report.append('<br/>'+" ".join([ "%5d" % x for x in order[0]])+(': <b> %f</b>'% order[1]))
            report.append('</div><br/>')## ordering
            report.append("Subgroups: "+" / ".join(['<a href="#%s"> %s </a>' % (node_ref(x),node_ref(x)) for x in grp.group if not x.type=="marker"])+"<br/>\n")
            report.append('</div>')## group class
        children=newchildren        
    report.append('</div>') ## groups section
    report.write(open(url,'w'))
    return

                               
